"""Living Room.

Sides:
- FORWARD: safe
- RIGHT: door to the hallway
- BACKWARD: window
- LEFT: window
"""

import random

from thehouse.characters import Monster
from thehouse.helpers import print_pause, validate_input, validate_text_input
from thehouse.helpers.constants import HOUSE_KEY_3, KNIFE, PASSEPARTOUT

from .room import Room


class Livingroom(Room):
    """Livingroom."""

    def __init__(self, player, thehouse):
        """Initialize class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        super().__init__(player, thehouse)
        self.monster = Monster(self.player)
        self.tries = random.randint(3, 6)
        self.safe_open = False
        self.safe_combination = random.randint(1, 9999)

    def blueprint(self) -> None:
        """Print blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is a painting.",
                "- On your right, there is a door.",
                "- Behind you, there is a window.",
                "- On your left, there is another window.",
            ]
        )

    def center(self):
        """Print welcome message."""
        if self.monster.is_alive:
            print_pause(
                [
                    "There is a terrible monster!",
                    "It is half human and half something indescribable!",
                    "Luckily, it is slow, but one of its tentacles tries to grab you!",
                ]
            )
            return self.fight_or_escape()
        else:
            print_pause("You are in the living room!")
            self.blueprint()
            return self.move()

    def fight_or_escape(self):
        """Let user choose between fighting or escaping."""
        choice = validate_input(
            "Do you want to fight or escape?",
            ["fight", "escape"],
        )

        if choice == "fight":
            return self.fight()
        else:
            return self.escape()

    def fight(self):
        """Let the user fight the monster."""
        if self.monster.is_alive and self.player.is_alive:
            print_pause("It is your turn to deal damage!")

            if KNIFE not in self.player.items:
                damage = 1
                print_pause("It seems like you need something to deal more damage!")
            else:
                damage = random.randint(2, 4)

            print_pause(f"You deal {damage} damage.")
            self.monster.lose_health(damage)

            if self.monster.is_alive:
                print_pause("It is the monster's turn to deal damage!")
                self.monster.deal_damage()

                if self.player.is_alive:
                    return self.fight_or_escape()
                else:
                    return str(self)
            else:
                print_pause(
                    [
                        "You successfully kill the monster!",
                        "The monster drops a key.",
                    ]
                )
                self.player.pick_an_item(PASSEPARTOUT)
                self.thehouse.rooms["studio"].door_locked = False
                return self.center()
        return str(self)

    def escape(self):
        """Let the user try to escape the monster."""
        print_pause("You are trying to escape the monster!")

        choice = validate_input(
            "Quick, choose a path!",
            ["Path 1", "Path 2", "Path 3", "Path 4", "Path 5", "Path 6"],
        )

        # Get the digit from "Path X"
        path_num = int(choice.split()[-1])

        if path_num == random.randint(1, 6):
            print_pause("You successfully escape the monster!")
            return "hallway"
        else:
            print_pause("You panic and cannot escape the fight.")
            self.tries -= 1

            if self.tries <= 0:
                print_pause("The monster reaches you!")
                self.monster.deal_damage()
                self.tries = random.randint(3, 6)

                if self.player.is_alive:
                    return self.fight_or_escape()
                else:
                    return str(self)
            else:
                return self.fight_or_escape()

    def right(self):
        """Move the user towards the hallway."""
        print_pause("You open the door and enter the hallway.")
        return "hallway"

    def backward(self):
        """Move the user towards the window."""
        print_pause(
            [
                "There is a window and you look outside.",
                "There is a car! Maybe you can use it to escape from this house!",
                "You go back.",
            ]
        )
        return str(self)

    def left(self):
        """Move the user towards left."""
        print_pause(
            [
                "There is a balcony.",
                "Outside, there is a garden.",
                "The low lights make the garden look bleak.",
                "You go back.",
            ]
        )
        return str(self)

    def forward(self):
        """Move the user towards the safe."""
        if self.safe_open:
            print_pause(
                [
                    "You already opened the safe and picked THE HOUSE KEY 3.",
                    "You go back.",
                ]
            )
            return str(self)
        else:
            print_pause(
                [
                    "There is a painting...",
                    "You look closely and see there is something behind it!",
                    "You move the painting and reveal a safe!",
                    "Maybe you can break it open!",
                ]
            )

            return self.break_open()

    def break_open(self):
        """Let the user break open the safe."""
        print_pause("You need a combination!")

        while True:
            choice = validate_text_input(
                "Enter the combination (4 digits), or type 'back': "
            )

            if choice.lower() == "back":
                return str(self)
            else:
                if choice == str(self.safe_combination).zfill(4) or choice == str(self.safe_combination):
                    print_pause(
                        ["You successfully open the safe!", "Inside, there is a key!"]
                    )
                    self.player.pick_an_item(HOUSE_KEY_3)
                    self.safe_open = True

                    return str(self)
                else:
                    print_pause("The combination is wrong! Try again!")



"""This file is responsible of exporting all rooms."""
from .bedroom import Bedroom
from .diningroom import Diningroom
from .hall import Hall
from .hallway import Hallway
from .kitchen import Kitchen
from .livingroom import Livingroom
from .studio import Studio

__all__ = [
    "Bedroom",
    "Diningroom",
    "Hall",
    "Hallway",
    "Kitchen",
    "Livingroom",
    "Studio",
]



"""Helpers.

This file will export all available helpers.
"""
from .print_pause import print_pause
from .random_death import random_death
from .validate_input import validate_input
from .validate_text_input import validate_text_input

__all__ = [
    "print_pause",
    "random_death",
    "validate_input",
    "validate_text_input",
]



"""Kitchen.

Sides:
- FORWARD: window
- RIGHT: door to the hall
- BACKWARD: drawers
- LEFT: drawers
"""

import random

from thehouse.helpers import print_pause, validate_input
from thehouse.helpers.constants import KNIFE

from .room import Room


class Kitchen(Room):
    """Kitchen."""

    def __init__(self, player, thehouse):
        """Initialize class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        super().__init__(player, thehouse)
        self.knife_in_drawer = random.randint(1, 3)
        self.knife_on_wall = random.choice(["left", "backward"])

    def blueprint(self) -> None:
        """Print the blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is a window.",
                "- On your right, there is a door.",
                "- Behind you, there are some kitchen drawers.",
                "- On your left, there are more kitchen drawers.",
            ]
        )

    def center(self):
        """Print a welcome message."""
        print_pause("You are in the kitchen!")
        self.blueprint()
        return self.move()

    def forward(self):
        """Print the content of the front side of the room."""
        print_pause(
            [
                "You look out of the window.",
                "There is nothing but darkness to see...",
                "You go back.",
            ]
        )
        return str(self)

    def right(self):
        """Move the player to the hall."""
        print_pause("You open the door and enter the hall.")
        return "hall"

    def _search_drawers(self, wall):
        print_pause(
            ["There are three drawers here. You decide to check them all."]
        )
        return self.pick_a_drawer(wall)

    def left(self):
        """Show user the drawers."""
        return self._search_drawers("left")

    def backward(self):
        """Show user the drawers."""
        return self._search_drawers("backward")

    def pick_a_drawer(self, wall):
        """Let the user pick a drawer.

        :param wall: the wall where the knife is located.
        """
        drawer_options = {
            "Open the 1st drawer": 1,
            "Open the 2nd drawer": 2,
            "Open the 3rd drawer": 3,
            "Nevermind...": "back",
        }

        while True:
            choice_label = validate_input(
                "Which drawer do you want to open?",
                list(drawer_options.keys()),
            )

            # Get the internal value from the mapping
            selected_key = next(k for k in drawer_options if k.lower() == choice_label)
            choice = drawer_options[selected_key]

            if choice == "back":
                print_pause("You go back to the center of the room.")
                return str(self)
            else:
                print_pause("You dig through the kitchen tools, looking for something useful.")

                if wall == self.knife_on_wall and choice == self.knife_in_drawer:
                    if KNIFE in self.player.items:
                        print_pause(
                            [
                                "You already have the knife from this drawer.",
                                "There is nothing else inside.",
                            ]
                        )
                    else:
                        print_pause("You find a KNIFE!")
                        self.player.pick_an_item(KNIFE)
                else:
                    print_pause("You find nothing but old silverware and rusty tools.")



"""UI components."""
from rich.layout import Layout
from rich.panel import Panel
from rich.text import Text
from rich.table import Table
from .state import GameState

def get_appbar():
    """Create a persistent top appbar."""
    if not GameState.player:
        return Panel(Text("Loading...", style="dim"), border_style="bright_magenta")

    # Health bar with hearts
    health_stars = "❤" * GameState.player.health
    pt_lost = "o" * (GameState.player.max_health - GameState.player.health)
    
    health_text = Text("HP: ", style="bold")
    health_text.append(health_stars, style="red")
    health_text.append(pt_lost, style="dim white")

    # Items
    items_text = Text(" 📦 Items: ", style="bold")
    if not GameState.player.items:
        items_text.append("None", style="dim italic")
    else:
        items_text.append(", ".join(GameState.player.items), style="yellow")

    # Room
    room_text = Text(f" 🏠 {GameState.current_room.upper()}", style="bold cyan")

    # Create a grid for the appbar to align elements
    table = Table.grid(expand=True)
    table.add_column(justify="left", ratio=1)
    table.add_column(justify="center", ratio=2)
    table.add_column(justify="right", ratio=1)
    
    table.add_row(health_text, items_text, room_text)
    
    return Panel(table, border_style="bright_magenta", padding=(0, 1))

def make_layout(main_content):
    """Create the full screen layout with appbar at the top."""
    layout = Layout()
    layout.split(
        Layout(name="header", size=3),
        Layout(name="main")
    )
    layout["header"].update(get_appbar())
    layout["main"].update(main_content)
    return layout



"""HALL.

Sides:
- FORWARD: the main door of the house.
- RIGHT: door to the diningroom.
- BACKWARD: hallway.
- LEFT: kitchen.
"""

from thehouse.helpers import print_pause
from thehouse.helpers.constants import HOUSE_KEY_1, HOUSE_KEY_2, HOUSE_KEY_3

from .room import Room


class Hall(Room):
    """Hall."""

    def blueprint(self) -> None:
        """Print blueprint of the house."""
        print_pause(
            [
                "- In front of you there's the main door of the house.",
                "- On your right there's a door.",
                "- Backwards there's the hallway.",
                "- On your left there's another door.",
            ]
        )

    def center(self):
        """Print welcome message."""
        print_pause("You're in the hall!")
        self.blueprint()
        return self.move()

    def backward(self):
        """Move the player to the hallway."""
        return "hallway"

    def left(self):
        """Move the player to the kitchen."""
        print_pause("You open the door and enter the room.")
        return "kitchen"

    def right(self):
        """Move the player to the diningroom."""
        print_pause("You open the door and enter the room.")
        return "diningroom"

    def forward(self):
        """Move the player towards the main door of the house."""
        if (
            HOUSE_KEY_1 in self.player.items
            and HOUSE_KEY_2 in self.player.items
            and HOUSE_KEY_3 in self.player.items
        ):
            print_pause("You unlock the door and finally exit the house!")
            self.player.escaped = True
        else:
            print_pause(
                ["You need three keys to open the door!", "You go back."]
            )

        return str(self)



"""Dining Room.

Sides:
- FORWARD: corpse
- RIGHT: corpse
- BACKWARD: corpse
- LEFT: door to the hall
"""
import random

from thehouse.helpers import print_pause
from thehouse.helpers.constants import HOUSE_KEY_2

from .room import Room


class Diningroom(Room):
    """Diningroom."""

    def __init__(self, player, thehouse):
        """Initialize class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        super().__init__(player, thehouse)
        self.key_in_corpse = random.choice(["forward", "left", "backward"])

    def blueprint(self) -> None:
        """Print the blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is a corpse!",
                "- On your right, there is another corpse!",
                "- Behind you, there is a third corpse!",
                "- On your left, there is a door.",
            ]
        )

    def center(self):
        """Print a welcome message."""
        print_pause(
            [
                "You are in the dining room!",
                "There is a bloody mess here...",
                "Someone or something killed three people!",
            ]
        )
        self.blueprint()
        return self.move()

    def forward(self):
        """Describe the corpse."""
        print_pause(
            [
                "Something smashed its head!",
                "There is a lot of blood and a strange material on the corpse.",
                "You check its pockets.",
            ]
        )

        if self.key_in_corpse == "forward":
            if HOUSE_KEY_2 in self.player.items:
                print_pause(
                    ["You already checked its pockets and found a key.", "You go back."]
                )
            else:
                print_pause("You find a key!")
                self.player.pick_an_item(HOUSE_KEY_2)
        else:
            print_pause("There is nothing inside its pockets. You go back.")

        return str(self)

    def right(self):
        """Describe the corpse."""
        print_pause(["Something ripped its arms off!", "You check its pockets."])

        if self.key_in_corpse == "left":
            if HOUSE_KEY_2 in self.player.items:
                print_pause(
                    ["You already checked its pockets and found a key.", "You go back."]
                )
            else:
                print_pause("You find a key!")
                self.player.pick_an_item(HOUSE_KEY_2)
        else:
            print_pause("There is nothing inside its pockets. You go back.")

        return str(self)

    def backward(self):
        """Describe the corpse."""
        print_pause(
            [
                "There is a huge hole inside the chest of the corpse.",
                "Something ripped its heart out!",
                "You check its pockets.",
            ]
        )

        if self.key_in_corpse == "backward":
            if HOUSE_KEY_2 in self.player.items:
                print_pause(
                    ["You already checked its pockets and found a key.", "You go back."]
                )
            else:
                print_pause("You find a key!")
                self.player.pick_an_item(HOUSE_KEY_2)
        else:
            print_pause("There is nothing inside its pockets. You go back.")

        return str(self)

    def left(self):
        """Move player towards the hall."""
        print_pause("You open the door and enter the hall.")
        return "hall"



"""Game state manager."""

class GameState:
    player = None
    current_room = "Bedroom"

    @classmethod
    def update(cls, player=None, room=None):
        if player:
            cls.player = player
        if room:
            cls.current_room = room



"""Player."""

from thehouse.helpers.constants import *
from thehouse.helpers.print_pause import print_pause

from .character import Character


class Player(Character):
    """Player.

    This class creates a concrete player.
    """

    def __init__(self):
        """Initialize escaped and items."""
        super().__init__()
        self.escaped = False
        self.items = []

    def __str__(self) -> str:
        """Return the health of the character."""
        return f"Player - health: {self.health}; escaped {self.escaped}"

    def __contains__(self, item) -> bool:
        """Return whether an item is in the items list.

        :param item: an item to check.
        """
        return item in self.items

    def escape_the_house(self) -> None:
        """Let the Player escape the house."""
        self.escaped = True

    def pick_an_item(self, item) -> None:
        """Pick an item and append it to the list.

        :param item: an item as a string.
        """
        self.items.append(item)
        print_pause(f"You pick {item}!")

    def print_items(self) -> None:
        """Print a list of items."""
        if not self.items:
            print_pause("Your pockets are empty!")
        else:
            for item in self.items:
                print_pause(f"- {item}")



"""Monster."""
import random

from .character import Character


class Monster(Character):
    """Monster class."""

    def __init__(self, player):
        """Inizialize the class with a player.

        :param player: the instantiated Player class.
        """
        super().__init__()
        self.player = player

    def __str__(self) -> str:
        """Return a string containing the health of the monster."""
        return f"Monster - health: {self.health}"

    def deal_damage(self) -> None:
        """Deal a random damage."""
        damage = random.randint(1, 2)
        self.player.lose_health(damage)



"""print_pause.

This function will print a message and then it will pause the terminal
"""

import sys
import time
from pyfiglet import Figlet
from rich.console import Console
from rich.panel import Panel
from rich.text import Text
from rich.live import Live
from .ui import make_layout

console = Console()

def print_pause(content, sleep=1):
    """Print a message (or multiple) and pause the terminal with an appbar.

    :param content: a string or a list of strings to print.
    :param sleep: the amount of seconds the terminal should sleep as integer.
    """
    try:
        # Normalize content to a list
        if isinstance(content, (str, Text)):
            lines = [content]
        else:
            lines = content

        # Check if we are dealing with a banner
        is_banner = any("\n" in str(line).strip() for line in lines)

        if is_banner:
            for line in lines:
                # Banners are shown above the appbar
                console.print(make_layout(line))
                time.sleep(sleep)
        else:
            # Display lines one by one inside the same box
            accumulated_text = Text()
            
            # Live display makes it look truly persistent during the pause
            with Live(make_layout(Panel(accumulated_text, border_style="bright_blue", padding=(1, 2))), 
                      console=console, refresh_per_second=4, transient=True) as live:
                for i, line in enumerate(lines):
                    if i > 0:
                        accumulated_text.append("\n\n")
                    
                    accumulated_text.append(str(line))
                    live.update(make_layout(Panel(accumulated_text, border_style="bright_blue", padding=(1, 2))))
                    time.sleep(sleep)
            
            # Final print to keep it in terminal history
            console.print(make_layout(Panel(accumulated_text, border_style="bright_blue", padding=(1, 2))))

    except KeyboardInterrupt:
        f = Figlet(font="catwalk")
        banner = f.renderText("BYE")
        console.print(Text(banner, style="bold yellow"))
        sys.exit(0)



"""Room blueprint."""
from thehouse.helpers import print_pause, validate_input


class Room:
    """Room blueprint."""

    def __init__(self, player, thehouse):
        """Initialize the class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        self.player = player
        self.thehouse = thehouse

    def __str__(self):
        """Return the name of the room."""
        return self.__class__.__name__.lower()

    def blueprint(self) -> None:
        """Print all sides of the room.

        This method is called if the user chooses 'help'.
        """
        pass

    def right(self) -> str:
        """Print the content of the right side of the room."""
        return str(self)

    def left(self) -> str:
        """Print the content of the left side of the room."""
        return str(self)

    def backward(self) -> str:
        """Print the content of the back side of the room."""
        return str(self)

    def forward(self) -> str:
        """Print the content of the front side of the room."""
        return str(self)

    def move(self) -> str:
        """Let the user move inside or outside the room."""
        # Map descriptive labels to move actions
        move_options = {
            "Go forward": "forward",
            "Go right": "right",
            "Go backward": "backward",
            "Go left": "left",
            "Check the room (Help)": "help",
            "Check your pockets (Items)": "items",
        }

        while True:
            choice_label = validate_input(
                "Where do you want to go?",
                list(move_options.keys()),
            )

            # Get the internal value from the mapping
            selected_key = next(k for k in move_options if k.lower() == choice_label)
            choice = move_options[selected_key]

            if choice == "right":
                return self.right()
            elif choice == "left":
                return self.left()
            elif choice == "backward":
                return self.backward()
            elif choice == "forward":
                return self.forward()
            elif choice == "help":
                self.blueprint()
            elif choice == "items":
                self.player.print_items()



"""thehouse.

This class wraps all rooms and play the game
"""
import random

from pyfiglet import Figlet
from rich.text import Text

import thehouse.rooms as rooms
from thehouse.helpers import print_pause, random_death
from thehouse.helpers.state import GameState


class TheHouse:
    """TheHouse."""

    def __init__(self, player):
        """Inizialize class.

        :param player: the instantiated player class.
        """
        self.player = player

        # Rooms
        self.rooms = {
            "bedroom": rooms.Bedroom(self.player, self),
            "diningroom": rooms.Diningroom(self.player, self),
            "hall": rooms.Hall(self.player, self),
            "hallway": rooms.Hallway(self.player, self),
            "kitchen": rooms.Kitchen(self.player, self),
            "livingroom": rooms.Livingroom(self.player, self),
            "studio": rooms.Studio(self.player, self),
        }

        self.current_room = "bedroom"
        # Initial state update
        GameState.update(player=self.player, room=self.current_room)

    def introduction(self):
        f = Figlet(font="catwalk")
        banner_text = f.renderText("THE HOUSE")
        banner = Text(banner_text, style="bold dim green")
        print_pause(banner, 3)

        print_pause([
            "Someone, or something hit you and you faint.",
            "You hear that this someone or something drags you to someplace.",
            "You open your eyes and find yourself lying on the floor.",
            "Your head is pounding and it hurts.",
            "With a lot of effort you stand up."
        ], 2)

    def play(self):
        """Play engine."""
        self.introduction()

        while not self.player.escaped and self.player.is_alive:
            # Update state before entering room
            GameState.update(room=self.current_room)
            
            # a room's center() method returns the next room
            next_room = self.rooms[self.current_room].center()
            self.current_room = next_room

            # Check if player is still alive
            if not self.player.is_alive:
                random_death()
                print_pause(Text("\n=== GAME OVER ===\n", style="bold red"), 3)
                break

            # Check if player has escaped
            if self.player.escaped:
                f = Figlet(font="catwalk")
                victory_text = f.renderText("VICTORY")
                victory = Text(victory_text, style="bold yellow")
                print_pause(victory)
                break



"""validate_input.

This function will validate input.
"""
import sys
import questionary
from pyfiglet import Figlet
from rich.text import Text
from rich.console import Console
from rich.panel import Panel
from .ui import get_appbar

console = Console()

def validate_input(prompt, options):
    """Validate the input from the user using questionary."""
    try:
        # Show appbar before the prompt
        console.print(get_appbar())

        choices = []
        for opt in options:
            choices.append(opt.capitalize() if isinstance(opt, str) else str(opt))

        if "quit" not in options:
            choices.append("Quit")

        answer = questionary.select(
            prompt,
            choices=choices,
            style=questionary.Style([
                ('qmark', 'fg:#673ab7 bold'),
                ('question', 'bold'),
                ('answer', 'fg:#f44336 bold'),
                ('pointer', 'fg:#673ab7 bold'),
                ('highlighted', 'fg:#673ab7 bold'),
                ('selected', 'fg:#cc2722'),
                ('separator', 'fg:#cc5454'),
                ('instruction', ''),
                ('text', ''),
                ('disabled', 'fg:#858585 italic')
            ])
        ).ask()

        if answer is None or answer == "Quit":
            f = Figlet(font="catwalk")
            goodbye_text = f.renderText("BYE")
            console.print(Text(goodbye_text, style="bold yellow"))
            sys.exit(0)

        result = answer.lower()
        if result in options:
            return result
        return result

    except (EOFError, PermissionError):
        console.print(Panel(
            "[bold red]Error: Interactive terminal required.[/bold red]\n\n"
            "This game requires an interactive terminal to run.\n"
            "If you are using Docker, please run it with the [bold]-it[/bold] flags:\n\n"
            "  [bold cyan]docker run -it dcdavidev/thehouse[/bold cyan]",
            title="System Error",
            border_style="red"
        ))
        sys.exit(1)
    except KeyboardInterrupt:
        f = Figlet(font="catwalk")
        goodbye_text = f.renderText("BYE")
        console.print(Text(f"\n{goodbye_text}", style="bold yellow"))
        sys.exit(0)



"""Game-wide constants for item names and other shared values."""

HOUSE_KEY_1 = "THE HOUSE KEY 1"
HOUSE_KEY_2 = "THE HOUSE KEY 2"
HOUSE_KEY_3 = "THE HOUSE KEY 3"
STUDIO_KEY = "THE STUDIO KEY"
PASSEPARTOUT = "PASSEPARTOUT"
KNIFE = "KNIFE"



"""Bedroom.

Sides:
- FORWARD: dresser
- RIGHT: window
- BACKWARD: bed
- LEFT: door to the hallway
"""

import random

from thehouse.helpers import print_pause, validate_input
from thehouse.helpers.constants import HOUSE_KEY_1

from .room import Room


class Bedroom(Room):
    """Bedroom."""

    def __init__(self, player, thehouse):
        """Initialize the class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        super().__init__(player, thehouse)
        self.key_in_drawer = random.randint(1, 5)

    def blueprint(self) -> None:
        """Print the blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is a dresser.",
                "- On your right, there is a window.",
                "- Behind you, there is a bed.",
                "- On your left, there is a door.",
            ]
        )

    def center(self):
        """Print welcome message."""
        print_pause("You are in the bedroom!")
        self.blueprint()
        return self.move()

    def left(self):
        """Move player to the left side of the room."""
        print_pause("You open the door and enter the hallway.")
        return "hallway"

    def backward(self):
        """Move player to the back side of the room."""
        print_pause(["You look tired.", "Do you want to rest a little?"])

        choice = validate_input("Type yes or no: ", ["yes", "no"])

        if choice == "yes":
            print_pause(["You decide to rest.", ".", ".", "."], 2)
            self.player.restore_health()
        else:
            print_pause("You go back.")

        return str(self)

    def right(self):
        """Print content of the right side of the room."""
        print_pause(
            [
                "You look out the window.",
                "Outside, it is pitch black!",
                "Something is moving in the darkness.",
                "It moves so fast that you can barely see it...",
                "You wonder how you can escape this house.",
                "And if it is even safe outside...",
                "You go back.",
            ],
            3,
        )

        return str(self)

    def forward(self):
        """Print content of the front side of the room."""
        print_pause(["There are five drawers."])
        return self.pick_a_drawer()

    def pick_a_drawer(self):
        """Let the user pick a drawer."""
        # Mapping labels to drawer numbers
        drawer_options = {
            "Open 1st drawer": 1,
            "Open 2nd drawer": 2,
            "Open 3rd drawer": 3,
            "Open 4th drawer": 4,
            "Open 5th drawer": 5,
            "You give up": "back",
        }

        while self.player.is_alive:
            choice_label = validate_input(
                "Choose wisely: ",
                list(drawer_options.keys()),
            )

            # Get the internal value from the mapping
            selected_key = next(k for k in drawer_options if k.lower() == choice_label)
            choice = drawer_options[selected_key]

            if choice == "back":
                print_pause("You go back to the center of the room.")
                return str(self)
            else:
                print_pause(
                    "There are some clothes in it. You dig through them to find something..."
                )

                if choice == self.key_in_drawer:
                    if HOUSE_KEY_1 in self.player.items:
                        print_pause(
                            [
                                "You already found a key in this drawer!",
                                "There is nothing more in it.",
                            ]
                        )
                    else:
                        print_pause("You find a key!")
                        self.player.pick_an_item(HOUSE_KEY_1)
                else:
                    print_pause(
                        [
                            "There is nothing among the clothes.",
                            "Something outside is moving...",
                        ]
                    )
                    self.player.lose_health()
        return str(self)



"""This module is responsible of initializing the game."""
from thehouse.characters import Player
from thehouse.helpers import print_pause, validate_input
from thehouse.thehouse import TheHouse


def main():
    while True:
        player = Player()
        game = TheHouse(player)

        game.play()

        print_pause("Do you want to play again?")
        choice = validate_input("Type yes or no: ", ["yes", "no"])

        if choice == "no":
            break


if __name__ == "__main__":
    main()



"""Character blueprint."""
import random

from rich.text import Text

from thehouse.helpers import print_pause


class Character:
    """Character blueprint."""

    def __init__(self):
        """Set max_health and health."""
        self.max_health = random.randint(5, 10)
        self.health = self.max_health

    @property
    def is_alive(self) -> bool:
        """Return whether the character is alive or not."""
        return True if self.health > 0 else False

    def lose_health(self, damage=1) -> None:
        """Take an amount of damage and substract it to health.

        :param damage: the amount of damage as integer the character takes.
        """
        self.health -= damage
        # Sound the terminal bell ONLY on damage
        if damage > 0:
            print("\a", end="", flush=True)
        self.print_health()

    def restore_health(self) -> None:
        """Restore the health to the maximum health of the character."""
        self.health = self.max_health
        self.print_health()

    def print_health(self) -> None:
        """Print a bar with the current health."""
        # Use rich Text for health bar coloring
        health_stars = "*" * self.health
        pt_lost = "-" * (self.max_health - self.health)
        
        health_text = Text("Health: ", style="bold")
        health_text.append(health_stars, style="red")
        health_text.append(pt_lost, style="dim")
        
        print_pause(health_text)



"""validate_text_input.

This function will validate text input.
"""
import sys
import questionary
from pyfiglet import Figlet
from rich.text import Text
from rich.console import Console
from rich.panel import Panel

console = Console()

def validate_text_input(prompt):
    """Validate the text input from the user using questionary."""
    try:
        answer = questionary.text(prompt).ask()

        if answer is None:
            f = Figlet(font="catwalk")
            goodbye_text = f.renderText("BYE")
            console.print(Text(goodbye_text, style="bold yellow"))
            sys.exit(0)

        return answer
    except (EOFError, PermissionError):
        console.print(Panel(
            "[bold red]Error: Interactive terminal required.[/bold red]\n\n"
            "This game requires an interactive terminal to run.\n"
            "If you are using Docker, please run it with the [bold]-it[/bold] flags:\n\n"
            "  [bold cyan]docker run -it dcdavidev/thehouse[/bold cyan]",
            title="System Error",
            border_style="red"
        ))
        sys.exit(1)
    except KeyboardInterrupt:
        f = Figlet(font="catwalk")
        goodbye_text = f.renderText("BYE")
        console.print(Text(f"\n{goodbye_text}", style="bold yellow"))
        sys.exit(0)



"""Studio.

Sides:
- FORWARD: door to the hallway
- RIGHT: window
- BACKWARD: desk
- LEFT: shelf
"""

import random

from thehouse.helpers import print_pause, validate_input
from thehouse.helpers.constants import PASSEPARTOUT, STUDIO_KEY

from .room import Room


class Studio(Room):
    """Studio class."""

    def __init__(self, player, thehouse):
        """Initialize class.

        :param player: the instantiated Player class.
        :param thehouse: the instantiated TheHouse class.
        """
        super().__init__(player, thehouse)
        self.door_locked = True
        self.lights = random.choice([True, False])
        self.key_in_book = random.randint(1, 3)

    def prompt_light(self):
        """Ask the user to turn the lights on."""
        print_pause(
            [
                "You hear something lurking in the dark.",
                "You lose some health while turning the lights on.",
            ]
        )
        self.player.lose_health()
        self.lights = True

    def blueprint(self) -> None:
        """Print blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is a closed door.",
                "- On your right, there is a window.",
                "- Behind you, there is a desk with some papers on it.",
                "- On your left, there is a shelf with many books on it.",
            ]
        )

    def center(self) -> str:
        """Print center of the room."""
        lights = "on" if self.lights else "off"

        print_pause(f"You find yourself in a room with the lights {lights}.")

        if not self.lights:
            self.prompt_light()

        if self.player.is_alive:
            print_pause("You are in the middle of a studio.")
            self.blueprint()
            return self.move()

        return str(self)

    def right(self) -> str:
        """Print the content of the right side."""
        print_pause(
            [
                "Behind the desk, there is a window.",
                "You glance outside, but it is pitch black.",
                "You cannot see anything interesting here.",
                "You go back.",
            ]
        )
        return str(self)

    def backward(self) -> str:
        """Print the content of the back side."""
        combination = self.thehouse.rooms["livingroom"].safe_combination

        print_pause(
            [
                "The desk is full of papers.",
                "There is a note on the desk.",
                f"It says: {combination:04d}",
                "You go back.",
            ]
        )
        return str(self)

    def left(self) -> str:
        """Print the content of the left side."""
        print_pause(
            [
                "On your left, there is a shelf full of books.",
                "You run your finger along the dusty spines "
                "and quickly read the titles.",
                "There are so many books on this shelf.",
                "You wonder if you have ever read this many books.",
            ]
        )

        return self.pick_a_book()

    def book_divine_comedy(self):
        """Print content of the Divine Comedy book."""
        if self.key_in_book == 1 and (STUDIO_KEY not in self.player.items):
            self.pick_the_key()
        else:
            print_pause(
                [
                    "Amor, ch’a nullo amato amar perdona,",
                    "mi prese del costui piacer sì forte,",
                    "che, come vedi, ancor non m’abbandona",
                ]
            )

    def book_the_king_in_yellow(self):
        """Print content of The King in Yellow book."""
        if self.key_in_book == 2 and (STUDIO_KEY not in self.player.items):
            self.pick_the_key()
        else:
            print_pause(
                [
                    "for I knew that the King in Yellow",
                    "had opened his tattered mantle",
                    "and there was only God to cry to now.",
                ]
            )

    def book_arkhams_secrets(self):
        """Print content of Arkham's secrets book."""
        if self.key_in_book == 3 and (STUDIO_KEY not in self.player.items):
            self.pick_the_key()
        else:
            print_pause(
                [
                    "West of Arkham the hills rise wild,",
                    "and there are valleys with deep",
                    "woods that no axe has ever cut.",
                ]
            )

    def pick_the_key(self):
        """Add the key into the player's items."""
        print_pause("You open the book and a key falls onto the ground.")
        self.player.pick_an_item(STUDIO_KEY)

    def pick_a_book(self) -> str:
        """Let the user choose a book."""
        book_options = {
            "Divine Comedy": "1",
            "The King in Yellow": "2",
            "Arkham's Secrets": "3",
            "Nevermind...": "back",
        }

        while True:
            choice_label = validate_input(
                "Which book do you want to pick?",
                list(book_options.keys()),
            )

            selected_key = next(k for k in book_options if k.lower() == choice_label)
            choice = book_options[selected_key]

            if choice == "1":
                self.book_divine_comedy()
            elif choice == "2":
                self.book_the_king_in_yellow()
            elif choice == "3":
                self.book_arkhams_secrets()
            elif choice == "back":
                print_pause("You go back to the center of the room.")
                return str(self)

    def forward(self) -> str:
        """Print content of the front side of the house."""
        if not self.door_locked:
            print_pause("You open the door and enter the hallway.")
            return "hallway"
        else:
            print_pause(
                [
                    "There is a closed door in front of you.",
                    "Do you want to try to open it?",
                ]
            )

            choice = validate_input("Your choice:", ["yes", "no"])

            if choice == "yes":
                if (
                    STUDIO_KEY in self.player.items
                    or PASSEPARTOUT in self.player.items
                ):
                    self.door_locked = False
                    print_pause(
                        ["You use a key to unlock the door.", "You exit the studio."]
                    )
                    return "hallway"
                else:
                    print_pause(
                        [
                            "The door is locked.",
                            "It seems you need a key to open it!",
                            "You go back.",
                        ]
                    )
                    return str(self)
            else:
                print_pause(
                    [
                        "You hear a scratching sound from the other side of the door!",
                        "You instantly jump back in fear!",
                    ]
                )
                self.player.lose_health()
                return str(self)



"""random_death.

This function will print a random death from the DEATHS variable.
It uses the print_pause funtion
"""

import random
from rich.text import Text

from .print_pause import print_pause

DEATHS = [
    "Something utterly bad stabs you in the back. You died!",
    "A strange figure grabs you and kills you instantly!",
    "Something from the dark makes you so mad you die!",
]


def random_death():
    """Print a random death from DEATHS variable."""
    death_msg = random.choice(DEATHS)
    print_pause(Text(death_msg, style="red"), 3)
    print_pause(Text("\n\n=== GAME OVER ===\n\n", style="bold blink red"))



"""Hallway.

Sides:
- FORWARD: the hall.
- RIGHT: door to the bedroom.
- BACKWARD: door to the studio.
- LEFT: door to the livingroom.
"""

from thehouse.helpers import print_pause, validate_input
from thehouse.helpers.constants import PASSEPARTOUT

from .room import Room


class Hallway(Room):
    """Hallway."""

    def blueprint(self) -> None:
        """Print the blueprint of the room."""
        print_pause(
            [
                "- In front of you, there is the hall of the house.",
                "- On your right, there is a small table and a door.",
                "- Behind you, there is a door.",
                "- On your left, there is a door and two paintings.",
            ]
        )

    def center(self):
        """Print welcome message and blueprint."""
        print_pause("You are in the hallway.")
        self.blueprint()
        return self.move()

    def backward(self):
        """Print content of the back side of the room."""
        studio = self.thehouse.rooms["studio"]

        if studio.door_locked or PASSEPARTOUT not in self.player.items:
            print_pause(
                [
                    "The door is locked!",
                    "It seems you need to find a key.",
                    "You go back.",
                ]
            )
            return str(self)
        else:
            print_pause("You open the door and enter the studio.")
            return "studio"

    def forward(self):
        """Move player to the hall."""
        return "hall"

    def right(self):
        """Move player to the bedroom."""
        if PASSEPARTOUT not in self.player.items:
            print_pause(
                [
                    "There is a small table and a door.",
                    "What do you want to do?",
                ]
            )

            # Using descriptive labels for the radio boxes
            choices_map = {
                "Check the table": "table",
                "Open the door": "door",
            }

            choice_label = validate_input(
                "Your choice: ",
                list(choices_map.keys()),
            )

            # Get the internal value from the mapping
            selected_key = next(k for k in choices_map if k.lower() == choice_label)
            choice = choices_map[selected_key]

            if choice == "door":
                print_pause("You open the door and enter the bedroom.")
                return "bedroom"
            else:
                return self.table()
        else:
            print_pause("You open the door and enter the bedroom.")
            return "bedroom"

    def table(self):
        """Let the user check if there's something inside the table."""
        print_pause(
            ["You open the drawer and find that it is empty.", "You go back."]
        )
        return str(self)

    def left(self):
        """Move player to the livingroom."""
        print_pause("You open the door and enter the living room.")
        return "livingroom"
